@@ -14,7 +14,7 @@ import base64
 
 from . import Config
 from logging import debug
-from .Utils import encode_to_s3, time_to_epoch, deunicodise, decode_from_s3
+from .Utils import encode_to_s3, time_to_epoch, deunicodise, decode_from_s3, check_bucket_name_dns_support
 from .SortedDict import SortedDict
 
 import datetime
@@ -63,6 +63,7 @@ def sign_string_v2(string_to_sign):
 
     Useful for REST authentication. See http://s3.amazonaws.com/doc/s3-developer-guide/RESTAuthentication.html
     string_to_sign should be utf-8 "bytes".
+    and returned signature will be utf-8 encoded "bytes".
     """
     secret_key = Config.Config().secret_key
     signature = base64.encodestring(hmac.new(encode_to_s3(secret_key), string_to_sign, sha1).digest()).strip()
@@ -84,7 +85,7 @@ def sign_request_v2(method='GET', canonical_uri='/', params=None, cur_headers=No
                                'uploads', 'versionId', 'versioning',
                                'versions', 'website',
                                # Missing of aws s3 doc but needed
-                               'delete', 'cors']
+                               'delete', 'cors', 'restore']
 
     if cur_headers is None:
         cur_headers = SortedDict(ignore_case = True)
@@ -149,19 +150,24 @@ def sign_url_base_v2(**parms):
     debug("Expiry interpreted as epoch time %s", parms['expiry'])
     signtext = 'GET\n\n\n%(expiry)d\n/%(bucket)s/%(object)s' % parms
     param_separator = '?'
-    if content_disposition is not None:
+    if content_disposition:
         signtext += param_separator + 'response-content-disposition=' + content_disposition
         param_separator = '&'
-    if content_type is not None:
+    if content_type:
         signtext += param_separator + 'response-content-type=' + content_type
         param_separator = '&'
     debug("Signing plaintext: %r", signtext)
     parms['sig'] = s3_quote(sign_string_v2(encode_to_s3(signtext)), unicode_output=True)
     debug("Urlencoded signature: %s", parms['sig'])
-    url = "%(proto)s://%(bucket)s.%(host_base)s/%(object)s?AWSAccessKeyId=%(access_key)s&Expires=%(expiry)d&Signature=%(sig)s" % parms
-    if content_disposition is not None:
+    if check_bucket_name_dns_support(Config.Config().host_bucket, parms['bucket']):
+        url = "%(proto)s://%(bucket)s.%(host_base)s/%(object)s"
+    else:
+        url = "%(proto)s://%(host_base)s/%(bucket)s/%(object)s"
+    url += "?AWSAccessKeyId=%(access_key)s&Expires=%(expiry)d&Signature=%(sig)s"
+    url = url % parms
+    if content_disposition:
         url += "&response-content-disposition=" + s3_quote(content_disposition, unicode_output=True)
-    if content_type is not None:
+    if content_type:
         url += "&response-content-type=" + s3_quote(content_type, unicode_output=True)
     return url
 
